unit Visual;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  Tomas;

type
  TForm1 = class(TForm)
    procedure goCreateForm(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  TPosition3D =
    Record
    X: Single;
    Y: Single;
    Z: Single;
    Angle: Single;
    End;

  AGV = class(TomasShape)
    public
    AGV_Pos: TPosition3D;
    AGV_Dest: Single;
    AGV_Container: TomasShape;
    Published
    Procedure Process; Override;
    End;

  CameraControl = class(TomasElement)
    Published
    Procedure Process; Override;
    End;

  Carrier = class(TomasShape)
    public
    CAR_Pos: TPosition3D;
    CAR_Num: Integer;
    CAR_Container: TomasShape;
    Published
    Procedure Process; Override;
    End;

  Spreader = class(TomasShape)
    public
    SPR_Pos: TPosition3D;
    SPR_Num: Integer;
    Published
    Procedure Process; Override;
    End;


var
  Form1: TForm1;
  ThisAGV: Array[1..3] Of AGV;
  Container: Array[1..3] Of TomasShape;
  ThisCarrier: Array[1..3] Of Carrier;
  ThisSpreader: Array[1..3] Of Spreader;
  i: Integer;
  AGVReady: TomasQueue;
  CarrierReady: TomasQueue;
  CarrierStart: TomasQueue;
  MyCamera: CameraControl;
  TheBackGround: TPicture;

implementation

{$R *.DFM}

Procedure AGV.Process;

  Procedure DoRoute;
  Begin
    MoveTo(AGV_Pos.X,AGV_Pos.Y,100,0.5,3);
    Bend2d(0,-30,0,10,3,3);
    DMoveTo(15,0,-30,0.5,3);
    Bend2d(0,-60,0,10,3,3);
    MoveTo(40,GetPositionY,GetPositionZ,0.5,3);
    Bend2D(0,90,0,10,3,3);
    MoveTo(GetPositionX,GetPositionY,20,0.5,3);
    Bend2D(0,90,0,10,3,3);
    MoveTo(AGV_Dest,GetPositionY,GetPositionZ,0.5,3);
    Bend2D(0,45,0,10,3,3);
    DMoveTo(-20,0,20,0.5,3);
    Bend2D(0,90,0,10,3,3);
    DMoveTo(10,0,10,0.5,3);
  End;  {DoRoute}

Begin
  Show(AGV_Pos.X,AGV_Pos.Y,AGV_Pos.Z);
  DoRoute;
  AGV_Container.DisConnect;
  EnterQueue(AGVReady);
  Suspend;
End;

Procedure CameraControl.Process;
Var
  X,Y,Z: Single;
Begin
  While AGVReady.Length <> 3 Do
  Begin
    StandBy;
  End;
  SetScreenString('Carriers picking up containers');
  ThisCarrier[1].Resume(TNow);
  While CarrierReady.Length <> 3 Do
  begin
    StandBy;
  End;
  SetScreenString('Camera moving over terminal');
  X:=0;
  Y:=40;
  Z:=150;
  While Z>100 Do
  Begin
    Z:=Z-0.2;
    TomasCamera.SetPosition(X,Y,Z);
    Hold(1);
  End;
  Finish;
End;  {CameraControl.Process}

procedure Carrier.Process;
var
  OtherCarrier: Carrier;
  MyAGV: AGV;
  ZDist: Single;
Begin
  Show(CAR_Pos.X,CAR_Pos.Y,CAR_Pos.Z);
  Suspend;
  LeaveQueue(CarrierStart);
  MyAGV:=AGVReady.FirstElement;
  MyAGV.LeaveQueue(AGVReady);
  MoveTo(20,GetPositionY,GetPositionZ,0.15,5);
  If CarrierStart.Length>0 Then
  Begin
    OtherCarrier:=CarrierStart.FirstElement;
    OtherCarrier.Resume(TNow);
  End;
  ZDist:=GetPositionZ - MyAGV.GetPositionZ;
  MoveTo(MyAGV.GetPositionX + ZDist,GetPositionY,GetPositionZ,0.15,5);
  Bend2D(0,-45,0,10,3,3);
  MoveTo(MyAGV.GetPositionX,GetPositionY,MyAGV.GetPositionZ,0.15,5);
  CAR_Container:=MyAGV.AGV_Container;
  With ThisSpreader[CAR_Num] Do
  begin
    DisConnect;
    Start(TNow);
  End;
  Suspend;
  EnterQueue(CarrierReady);
  Suspend;
End;  {Carrier.Process}

Procedure Spreader.Process;
Begin
  DMoveTo(0,-5,0,0.15,1);
  ThisCarrier[SPR_Num].CAR_Container.ConnectWith(Self,0,-2,0);
  DMoveTo(0,5,0,0.15,1);
  ThisCarrier[SPR_Num].Resume(TNow);
  Suspend;
End;  {Spreader.Process}

Procedure DrawBackGround;
var
  I: Integer;
Begin
  For i:=-10 to 10 Do
  Begin
    DrawLine(10 * i,0,0,10 * i,0,150,clgray);
    DrawLine(-100,0,100 + 10 * i,100,0,100 + 10 *i,clGray);
  End;
  DrawLine(9.5,0,64,- 3.5,0,51,clWhite);
  DrawLine(-10.5,0,64,-23.5,0,51,clWhite);
  DrawLine(-30.5,0,64,-43.5,0,51,clWhite);
End;  {DrawBackGround}

procedure TForm1.GoCreateForm(Sender: TObject);
Var
  i: Integer;
begin
  SetAnimation;
  SetBackGround(DrawBackGround);
  TomasCamera.SetRotation(-30,0,0.0);
  TomasCamera.SetPosition(0,40,150);
  MyCamera:=CameraControl.Create('Camera');
  MyCamera.Start(0);
  For i:=1 To 3 Do
  Begin
    ThisAGV[i]:=AGV.Create('AGV');
    With ThisAGV[i] Do
    Begin
      AddBoxObject(2.5,2.0,18.6,rmSolidShade,clRed,0,0,0,0,0,0,0);
      AddBoxObject(0.5,0.5,1.0,rmSolidShade,clGreen,1,0,0,90,1.0,1.5,9.05);
      AddBoxObject(0.5,0.5,1.0,rmSolidShade,clGreen,1,0,0,90,-1.0,1.5,9.05);
      AddBoxObject(0.5,0.5,1.0,rmSolidShade,clGreen,1,0,0,90,-1.0,1.5,-9.05);
      AddBoxObject(0.5,0.5,1.0,rmSolidShade,clGreen,1,0,0,90,1.0,1.5,-9.05);
      AddCircleObject(1.0,rmSolid,clBlack,0,0,0,0,1.35,-0.5,7.0);
      AddCircleObject(1.0,rmSolid,clBlack,0,0,0,0,-1.35,-0.5,7.0);
      AddCircleObject(1.0,rmSolid,clBlack,0,0,0,0,-1.35,-0.5,-7.0);
      AddCircleObject(1.0,rmSolid,clBlack,0,0,0,0,1.35,-0.5,-7.0);
      AGV_Pos.X:=0;
      AGV_Pos.Y:=1.0;
      AGV_Pos.Z:=100 + 20 * i;
      AGV_Dest:=-40 + 20 * i;
      Start(0);
    End;
    Container[i]:=TomasShape.Create('Container');
    With Container[i] Do
    Begin
      AddBoxObject(2.0,2.0,12.0,rmSolidShade,clYellow,0,0,0,0,0,0,0);
      ConnectWith(ThisAGV[i],0,2.0,0);
      ThisAGV[i].AGV_Container:=Container[i];
    End;
  End;
  CarrierStart:=TomasQueue.Create('CarrierStart');
  For i:=1 To 3 Do
  Begin
    ThisCarrier[i]:=Carrier.Create('Carrier');
    With ThisCarrier[i] Do
    Begin
      AddBoxObject(3.5,1.0,18.6,rmSolidShade,clNavy,0,0,0,0,0,12,0);
      AddBoxObject(0.5,0.5,12.0,rmSolidShade,clBlue,1,0,0,90,1.75,6.0,8.0);
      AddBoxObject(0.5,0.5,12.0,rmSolidShade,clBlue,1,0,0,90,-1.75,6.0,8.0);
      AddBoxObject(0.5,0.5,12.0,rmSolidShade,clBlue,1,0,0,90,-1.75,6.0,-8.0);
      AddBoxObject(0.5,0.5,12.0,rmSolidShade,clBlue,1,0,0,90,1.75,6.0,-8.0);
      AddBoxObject(0.5,0.5,20.0,rmSolidShade,clNavy,0,0,0,0,-1.80,2.5,0);
      AddBoxObject(0.5,0.5,20.0,rmSolidShade,clNavy,0,0,0,0,1.80,2.5,0);
      AddCircleObject(1.5,rmSolid,clBlack,0,0,0,0,1.90,0.75,8.0);
      AddCircleObject(1.5,rmSolid,clBlack,0,0,0,0,-1.90,0.75,8.0);
      AddCircleObject(1.5,rmSolid,clBlack,0,0,0,0,-1.90,0.75,-8.0);
      AddCircleObject(1.5,rmSolid,clBlack,0,0,0,0,1.90,0.75,-8.0);
      CAR_Num:=i;
      CAR_Pos.X:=-20;
      CAR_Pos.Y:=0;
      CAR_Pos.Z:=80 +10 * i;
      EnterQueue(CarrierStart);
      Start(0);
    End;
    ThisSpreader[i]:=Spreader.Create('Spreader');
    ThisSpreader[i].SPR_Num:=i;
    ThisSpreader[i].AddBoxObject(3.0,0.5,12.0,rmSolidShade,clGreen,0,0,0,0,0,0,0);
    ThisSpreader[i].ConnectWith(ThisCarrier[i],0,10.5,0);
    ThisCarrier[i].Turn(0,90,0);
  End;
  AGVReady:=TomasQueue.Create('AGVReady');
  CarrierReady:=TomasQueue.Create('CarrierReady');
  SetScreenString(' 3 AGV''s moving with container');
  SetBackGroundColor(clSilver);
end;

end.
